import pathlib

import numpy as np
import qpimage
from skimage.external import tifffile

from . import search
from .roi import ROIManager


FILE_ROI_DATA_H5 = "roi_data.h5"
FILE_ROI_DATA_TIF = "roi_data.tif"
FILE_SLICES = "roi_slices.txt"


def extract_roi(h5series, dir_out, size_m, size_var=.5, max_ecc=.7,
                dist_border=10, pad_border=40, exclude_overlap=30,
                bg_amp_offset="mean", bg_amp_profile="ramp",
                bg_amp_border_px=5, bg_amp_border_perc=5,
                bg_pha_offset="mean", bg_pha_profile="ramp",
                bg_pha_border_px=5, bg_pha_border_perc=5,
                ret_roimgr=False):

    h5in = pathlib.Path(h5series)
    dout = pathlib.Path(dir_out)

    h5out = dout / FILE_ROI_DATA_H5
    imout = dout / FILE_ROI_DATA_TIF
    slout = dout / FILE_SLICES

    with qpimage.QPSeries(h5file=h5in, h5mode="r") as qps, \
            qpimage.QPSeries(h5file=h5out, h5mode="w") as qps_roi, \
            tifffile.TiffWriter(str(imout), imagej=True) as tf:
        rmgr = ROIManager(qps.identifier)
        for ii in range(len(qps)):
            qpi = qps[ii]
            # find objects
            slices = search.search_phase_objects(
                qpi=qpi,
                size_m=size_m,
                size_var=size_var,
                max_ecc=max_ecc,
                dist_border=dist_border,
                pad_border=pad_border,
                exclude_overlap=exclude_overlap)
            for jj, sl in enumerate(slices):
                # Write QPImage
                qpisl = qpi.__getitem__(sl)
                qpisl.compute_bg(which_data="amplitude",
                                 fit_offset=bg_amp_offset,
                                 fit_profile=bg_amp_profile,
                                 border_perc=bg_amp_border_perc,
                                 border_px=bg_amp_border_px)
                qpisl.compute_bg(which_data="phase",
                                 fit_offset=bg_pha_offset,
                                 fit_profile=bg_pha_profile,
                                 border_perc=bg_pha_border_perc,
                                 border_px=bg_pha_border_px)
                slident = "{}.{}".format(qpi["identifier"], jj)
                qps_roi.add_qpimage(qpisl, identifier=slident)
                rmgr.add(roislice=sl, image_index=ii,
                         roi_index=jj, identifier=slident)

        if len(qps_roi):
            # Write TIF
            # determine largest image
            sxmax = np.max([qq.shape[0] for qq in qps_roi])
            symax = np.max([qq.shape[1] for qq in qps_roi])
            dummy = np.zeros((2, sxmax, symax), dtype=np.float32)
            for qpir in qps_roi:
                dummy[0, :, :] = 0
                dummy[1, :, :] = 1
                res = 1 / qpir["pixel size"] * 1e-6  # use µm
                sx, sy = qpir.shape
                dummy[0, :sx, :sy] = qpir.pha
                dummy[1, :sx, :sy] = qpir.amp
                tf.save(data=dummy, resolution=(res, res, None))

    rmgr.save(slout)
    ret = h5out
    if ret_roimgr:
        ret = ret, rmgr
    return ret



import pathlib

import numpy as np
import qpformat
import qpimage
from skimage.external import tifffile


FILE_SENSOR_DATA_H5 = "sensor_data.h5"
FILE_SENSOR_DATA_TIF = "sensor_data.tif"


def convert(path_in, dir_out, bg_path=None, meta_data={}):

    path = pathlib.Path(path_in).resolve()
    dout = pathlib.Path(dir_out).resolve()

    h5out = dout / FILE_SENSOR_DATA_H5
    imout = dout / FILE_SENSOR_DATA_TIF

    ds = qpformat.load_data(path=path,
                            bg_path=bg_path,
                            meta_data=meta_data)

    if h5out.exists():
        with qpimage.QPSeries(h5file=h5out, h5mode="r") as qpsr:
            if ds.identifier == qpsr.identifier:
                create = False
            else:
                create = True
    else:
        create = True

    if create:
        # Write data
        with qpimage.QPSeries(h5file=h5out,
                              h5mode="w",
                              identifier=ds.identifier) as qps, \
                tifffile.TiffWriter(str(imout), imagej=True) as tf:
            for ii in range(len(ds)):
                # Get data
                if ds.is_series:
                    qpi = ds.get_qpimage(ii)
                else:
                    qpi = ds.get_qpimage()
                # Write QPImage
                imid = "{}:{}".format(ds.identifier, ii)
                qps.add_qpimage(qpi, identifier=imid)
                # Write TIF
                res = 1 / qpi["pixel size"] * 1e-6  # use µm
                dshape = (1, qpi.shape[0], qpi.shape[1])
                dataa = np.array(qpi.amp, dtype=np.float32).reshape(*dshape)
                datap = np.array(qpi.pha, dtype=np.float32).reshape(*dshape)
                data = np.vstack((datap, dataa))
                tf.save(data=data, resolution=(res, res, None))

    return h5out



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from os.path import dirname, realpath, exists
from setuptools import setup, find_packages
import sys


author = u"Paul Müller"
authors = [author]
description = 'user-friendly quantitative phase imaging analysis'
name = 'drymass'
year = "2017"

sys.path.insert(0, realpath(dirname(__file__))+"/"+name)
from _version import version

setup(
    name=name,
    author=author,
    author_email='dev@craban.de',
    url='https://github.com/RI-imaging/DryMass',
    version=version,
    packages=find_packages(),
    package_dir={name: name},
    license="MIT",
    description=description,
    long_description=open('README.rst').read() if exists('README.rst') else '',
    install_requires=["matplotlib",
                      "numpy",
                      "qpformat==0.1.1",
                      "qpimage==0.1.3",
                      "qpsphere==0.1.1",
                      "scikit-image>=0.13.1",
                      ],
    setup_requires=['pytest-runner'],
    tests_require=["pytest"],
    entry_points={
       "console_scripts": [
           "dm_analyze_sphere = drymass.__main__:cli_analyze_sphere",
           "dm_convert = drymass.__main__:cli_convert",
           "dm_extract_roi = drymass.__main__:cli_extract_roi",
            ],
       },
    python_requires='>=3.5, <4',
    keywords=["digital holographic microscopy",
              "optics",
              "quantitative phase imaging",
              "refractive index",
              "scattering",
              ],
    classifiers= [
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Intended Audience :: Science/Research'
                 ],
    platforms=['ALL'],
    )



def fbool(value):
    """boolean"""
    if isinstance(value, str):
        value = value.lower()
        if value == "false":
            value = False
        elif value == "true":
            value = True
        elif value:
            value = bool(float(value))
        else:
            raise ValueError("empty string")
    else:
        value = bool(float(value))
    return value


def fintlist(alist):
    """A list of integers"""
    outlist = []
    if isinstance(alist, str):
        # we have a string (comma-separated integers)
        alist = alist.strip().strip("[] ").split(",")
    for it in alist:
        if it:
            outlist.append(int(it))
    return outlist


def lcstr(astr):
    """lower-case string"""
    return astr.lower()


def tupletupleint(items):
    """A tuple containing x- and y- slice tuples"""
    if isinstance(items, str):
        for s in " ()[]":
            items = items.replace(s, "")
        if items.strip():
            # we have a string representation of the slices
            x1, x2, y1, y2 = items.split(",")
            out = ((int(x1), int(x2)),
                   (int(y1), int(y2)))
        else:
            out = ()
    else:
        if items:
            (x1, x2), (y1, y2) = items
            out = ((int(x1), int(x2)),
                   (int(y1), int(y2)))
        else:
            out = ()
    return out


func_types = {fbool: bool,
              fintlist: list,
              lcstr: str,
              tupletupleint: tuple,
              }



import numpy as np

from .cfg_funcs import fbool, lcstr, tupletupleint

config = {
    "bg": {
        # see qpimage library: e.g. fit, gauss, mean, mode
        "amplitude offset":
            ("mean", lcstr, "Amplitude background correction offset method"),
        # see qpimage library: e.g. ramp, offset
        "amplitude profile":
            ("ramp", lcstr, "Amplitude background correction profile method"),
        "amplitude border perc":
            (10, float, "Amplitude background border region to analyze [%]"),
        "amplitude border px":
            (5, int, "Amplitude background border region to analyze [px]"),
        # see qpimage library: e.g. fit, gauss, mean, mode
        "phase offset":
            ("mean", lcstr, "Phase background correction offset method"),
        # see qpimage library: e.g. ramp, offset
        "phase profile":
            ("ramp", lcstr, "Phase background correction profile method"),
        "phase border perc":
            (10, float, "Phase background border region to analyze [%]"),
        "phase border px":
            (5, int, "Phase background border region to analyze [px]"),
    },
    "roi": {
        "dist border":
            (10, int, "Minimum distance of objects to image border [px]"),
        "eccentricity max":
            (.7, float, "Allowed maximal eccentricity of the specimen"),
        "exclude overlap":
            (30., float, "Allowed distance between two objects [px]"),
        "force":
            ((), tupletupleint, "Force ROI coordinates (x1,x2,y1,y2) [px]"),
        "pad border":
            (40, int, "Padding of object regions [px]"),
        "size variation":
            (.5, float, "Allowed variation relative to specimen size"),
    },
    "meta": {
        "medium index":
            (np.nan, float, "Refractive index of the surrounding medium"),
        "pixel size um":
            (np.nan, float, "Detector pixel size [µm]"),
        "wavelength nm":
            (np.nan, float, "Imaging wavelength [nm]"),
    },
    "output": {
        "roi images":
            (True, fbool, "Rendered phase images with ROI location"),
        "sphere images":
            (True, fbool, "Phase/Intensity images for sphere analysis"),
    },
    "specimen": {
        "size um":
            (10, float, "Approximate diameter of the specimen [µm]"),
    },
    "sphere": {
        "method":
            ("edge", lcstr, "Method for determining sphere parameters"),
        "model":
            ("projection", lcstr, "Physical sphere model"),
        "edge coarse":
            (.4, float, "Coarse edge detection filter size"),
        "edge fine":
            (.1, float, "Fine edge detection filter size"),
        "edge clip radius min":
            (.9, float, "Interior edge point filtering radius"),
        "edge clip radius max":
            (1.1, float, "Exterior edge point filtering radius"),
        "edge iter":
            (20, int, "Maximum number iterations for coarse edge detection"),
        "refraction increment":
            (.18, float, "Refraction increment [mL/g]"),
        "radial inclusion factor":
            (1.2, float, "Radial inclusion factor for dry mass computation"),
    },
}
